@@ -6,13 +6,10 @@ from connectwrap import utils
 class db:
 
     # Constructor
-    def __init__(self, db_filepath, db_table):
+    def __init__(self, db_filepath, db_table = None):
         if (type(db_filepath) is not str):
             raise TypeError("The path to the db_filepath isn't a string!")
 
-        if (type(db_table) is not str):
-            raise TypeError("The db_table attribute isn't a string!")
-
         if (os.path.exists(db_filepath) == False):
             raise FileNotFoundError("The file that the db_filepath argument represents doesn't exist!")
 
@@ -22,8 +19,12 @@ class db:
         if (utils.isdb(db_filepath) == False):
             raise ValueError("The db_filepath argument doesn't have the correct extension! Use .db, .sqlite, or .sqlite3!")
 
+        if (db_table is not None):
+            if (type(db_table) is not str):
+                raise TypeError("The db_table attribute isn't a string!")
+            
         self.db_filepath = str(db_filepath)
-        self.db_table = str(db_table)
+        self.db_table = db_table
         self.connection = sqlite3.connect(self.db_filepath)
         self.connection_cursor = self.connection.cursor()
         self.connection_status = bool(True)
@@ -105,6 +106,9 @@ class db:
         if (type(db_table) is not str):
             raise TypeError("The db_table argument isn't a string!")
 
+        if (db.table_exists(self, db_table) == False):
+            raise db.TableNotFoundError("The db_table attribute table doesn't exist within the database!")
+
         self.db_table = db_table
 
     # Return the connection_status attribute representing whether the Database connection is open or closed. 
@@ -145,7 +149,7 @@ class db:
         connection = sqlite3.connect(self.db_filepath)
         connection.row_factory = sqlite3.Row
         connection_cursor = connection.cursor()
-        query = str("SELECT * FROM {0}").format(self.db_table)
+        query = str(f"SELECT * FROM {self.db_table}")
         connection_cursor.execute(query)
         row = connection_cursor.fetchone()
         connection.close()
@@ -166,7 +170,7 @@ class db:
             raise KeyError("The key argument doesn't exist within the db_table attribute table!")
 
         column_values = list([])
-        query = str("SELECT {0} FROM {1}").format(key, self.db_table)
+        query = str(f"SELECT {key} FROM {self.db_table}")
 
         for column in db.execute(self, query):
             column = str(column).strip("(,')")
@@ -231,7 +235,7 @@ class db:
 
         table = list([])
         keys = list(db.get_keys(self))
-        query = str("SELECT * FROM {0}").format(self.db_table)
+        query = str(f"SELECT * FROM {self.db_table}")
 
         for row in db.execute(self, query):
             row_dict = dict.fromkeys(keys)
@@ -262,6 +266,50 @@ class db:
 
         return table 
 
+    # Select and return a list of dictionaries representing rows in the db_table attribute table limited to the number designated in the 'total' argument.
+    def get_top(self, total):
+        if (self.connection_status != True):
+            raise db.DatabaseNotOpenError("Database is not open! The connection status attribute is not set to True!")
+
+        if (db.table_exists(self, self.db_table) == False):
+            raise db.TableNotFoundError("The db_table attribute table doesn't exist within the database!")
+
+        if (type(total) is not int):
+            raise TypeError("The total argument isn't an int!")
+
+        table = list([])
+        keys = list(db.get_keys(self))
+        query = str(f"SELECT * FROM {self.db_table} LIMIT {total}")
+
+        for row in db.execute(self, query):
+            row_dict = dict.fromkeys(keys)
+            row_factor = list([])
+            i = int(0)
+
+            for column in row:
+                column = str(column).strip("(,')") 
+                            
+                if (column.isdigit() == True):
+                    column = int(column)
+                elif (utils.isfloat(column) == True):
+                    column = float(column)   
+                elif (utils.ishex(column) == True):
+                    column = bytes.fromhex(column)    
+                elif (column == "None"):
+                    column = None
+                else:
+                    column = str(column)
+                
+                row_factor.append(column)
+
+            for key in row_dict:
+                row_dict[key] = row_factor[i]
+                i += 1
+                
+            table.append(row_dict)
+
+        return table
+
     # Rename db_table attribute table. 
     def rename_table(self, table):
         if (self.connection_status != True):
@@ -273,7 +321,7 @@ class db:
         if (db.table_exists(self, table) == True):
             raise db.TableExistsError("The tablename argument table already exists within the database!")
 
-        query = str("ALTER TABLE {0} RENAME TO {1}").format(self.db_table, table)
+        query = str(f"ALTER TABLE {self.db_table} RENAME TO {table}")
         db.execute(self, query)
         db.commit(self)
         self.db_table = table
@@ -289,7 +337,7 @@ class db:
         if (db.table_exists(self, table) == False):
             raise db.TableNotFoundError("The table doesn't exist within the database!")
         
-        query = str("DROP TABLE {0}").format(table)
+        query = str(f"DROP TABLE {table}")
         db.execute(self, query)
         db.commit(self)
 
@@ -312,8 +360,6 @@ class db:
         if (db.key_exists(self, key) == False):
             raise KeyError("The key argument doesn't exist within the db_table attribute table!")
 
-        query = str("DELETE FROM {0} WHERE {1}={2}")
-
         if (value == None):
             value = str("'None'")
             
@@ -323,7 +369,7 @@ class db:
         if (type(value) is str):
             value = str("'" + value + "'")
 
-        query = query.format(self.db_table, key, value)
+        query = str(f"DELETE FROM {self.db_table} WHERE {key}={value}")
         db.execute(self, query)
         db.commit(self)
 
@@ -341,7 +387,6 @@ class db:
         if (db.table_exists(self, table) == True):
             raise db.TableExistsError("The table already exists within the database!")
 
-        query = str("CREATE TABLE {0} ({1})")
         record = str("")
         count = int(0)
     
@@ -370,7 +415,7 @@ class db:
 
             count += 1
         
-        query = query.format(table, record)  
+        query = str(f"CREATE TABLE {table} ({record})") 
         db.execute(self, query)
         db.commit(self)
 
@@ -406,7 +451,7 @@ class db:
         else:
             datatype = "NULL"
 
-        query = str("ALTER TABLE {0} ADD {1} {2}").format(self.db_table, column, datatype)
+        query = str(f"ALTER TABLE {self.db_table} ADD {column} {datatype}")
         db.execute(self, query)
         db.commit(self)
 
@@ -428,6 +473,20 @@ class db:
 
         for row in db.get_table(self):
             print(self.db_table, "Row:", row)
+
+    # Select and output to terminal the rows from the db_table attribute table limited to the number designated in the 'total' argument.
+    def select_top(self, total):
+        if (self.connection_status != True):
+            raise db.DatabaseNotOpenError("Database is not open! The connection status attribute is not set to True!")
+
+        if (db.table_exists(self, self.db_table) == False):
+            raise db.TableNotFoundError("The db_table attribute table doesn't exist within the database!")
+
+        if (type(total) is not int):
+            raise TypeError("The total argument isn't an int!")
+
+        for row in db.get_top(self, total):
+            print(self.db_table, "Row:", row)
        
     # Select and output to terminal the values from keys within the db_table attribute table. 
     # Each arg in *args arguments must be strings containing key names within the table.
@@ -486,7 +545,6 @@ class db:
         if (db.table_exists(self, self.db_table) == False):
             raise db.TableNotFoundError("The db_table attribute table doesn't exist within the database!")
 
-        query = str("INSERT INTO {0} VALUES ({1})")
         record = str("")
         count = int(0)
 
@@ -517,7 +575,7 @@ class db:
 
             count += 1
 
-        query = query.format(self.db_table, record)
+        query = str(f"INSERT INTO {self.db_table} VALUES ({record})")
         db.execute(self, query)
         db.commit(self)
 
@@ -549,8 +607,6 @@ class db:
         if (db.key_exists(self, check_key) == False):
             raise KeyError("The check_key argument doesn't exist within the db_table attribute table!")
 
-        query = str("UPDATE {0} SET {1}={2} WHERE {3}={4}")
-
         if (change_value == None):
             change_value = str("'None'")
             
@@ -569,7 +625,7 @@ class db:
         if (type(check_value) is str):
             check_value = str("'" + check_value + "'")
 
-        query = query.format(self.db_table, change_key, change_value, check_key, check_value)
+        query = str(f"UPDATE {self.db_table} SET {change_key}={change_value} WHERE {check_key}={check_value}")
         db.execute(self, query)
         db.commit(self)
 
@@ -595,7 +651,7 @@ class db:
         if (self.connection_status != True):
             raise db.DatabaseNotOpenError("Database is not open! The connection status attribute is not set to True!")
         
-        if (type(table) is not str):
+        if (type(table) is not str and table is not None):
             raise TypeError("The table argument isn't a string!")
 
         for name in db.get_tablenames(self):
